package net.sf.javaprinciples.core;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.bind.JAXBElement;

import org.apache.commons.codec.CharEncoding;
import org.apache.commons.io.IOUtils;
import org.springframework.core.io.Resource;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;
import org.springframework.xml.transform.StringSource;

public class JaxbResourceMarshaller<T> implements ResourceMarshaller<T>
{
    private Jaxb2Marshaller marshaller;

    public String loadResourceAsString(Resource resource)
    {
        String resourceString = null;
        InputStream input = null;
        try
        {
            input = resource.getInputStream();
            resourceString = IOUtils.toString(input, CharEncoding.UTF_8);
        }
        catch (IOException e)
        {
            throw new RuntimeException("Could not load input process request", e);
        }
        finally
        {
            IOUtils.closeQuietly(input);
        }
        return resourceString;
    }

    @SuppressWarnings("unchecked")
    public T unmarshall(String resource)
    {
        // Will result in the unmarshall to stop if it cant handle an element
        marshaller.setValidationEventHandler(new javax.xml.bind.helpers.DefaultValidationEventHandler());

        Object object = marshaller.unmarshal( new StringSource(resource) );

        if (object instanceof JAXBElement)
        {
            JAXBElement<T> element = ((JAXBElement<T>)object);
            return element.getValue();
        }
        else
        {
            return (T)object;
        }
    }

    @SuppressWarnings("unchecked")
    public T unmarshall(Resource resource)
    {
        String requestMessage = loadResourceAsString(resource);
        return (T) unmarshall(requestMessage);
    }

    public void setMarshaller(Jaxb2Marshaller marshaller)
    {
        this.marshaller = marshaller;
    }
}
